<?php
// Labels used for different portions of the plugin
$labels = array();
$labels['activate'] = 'Activar';
$labels['twofactor_webauthn'] = 'Autenticación de dos factores';
$labels['config'] = 'Autenticación de dos factores';
$labels['two_step_verification_form'] = 'Forma de autenticación de dos factores';
$labels['add_new'] = 'Añadir llave de Seguridad';
$labels['no_keys_yet'] = 'No hay llaves registradas';
$labels['registered_keys'] = 'Llaves de seguridad registradas';
$labels['loading'] = 'Cargando...';
$labels['confirm_delete_key'] = 'Eliminar la llave con ID';
$labels['key_registered'] = 'Llave registrada correctamente';
$labels['key_deleted'] = 'Llave borrada';
$labels['please_activate_key'] = 'Por favor activa tu llave de seguridad';
$labels['verifying_key'] = 'Verificando llave...';
$labels['authentication_succeeded'] = 'Autenticación correcta';
$labels['authentication_failed'] = 'Autenticación fallida';
$labels['authentication_cancelled'] = 'Autenticacion fallida';
$labels['test_key'] = 'Probar llave';
$labels['key_checked'] = 'Llave verificada correctamente';

// Messages used for the different portions of the plugin
$messages = array();
$messages['successfully_saved'] = 'Ajustes guardados correctamente.';
