<?php
// Labels used for different portions of the plugin
$labels = array();
$labels['activate'] = 'Activer';
$labels['twofactor_webauthn'] = 'Authentification U2F';
$labels['config'] = 'Authentification U2F';
$labels['two_step_verification_form'] = 'Formulaire U2F';
$labels['add_new'] = 'Ajouter une clé de sécurité';
$labels['no_keys_yet'] = 'Pas de clés encore enregistrée';
$labels['registered_keys'] = 'Clé(s) de securité enregistrée(s)';
$labels['loading'] = 'Chargement...';
$labels['confirm_delete_key'] = 'Vraiement détruire la clé avec ID';
$labels['key_registered'] = 'Clé enregistrée avec succès';
$labels['key_deleted'] = 'Clé détruite';
$labels['please_activate_key'] = 'Veuillez activer votre clé de sécurité maintenant';
$labels['verifying_key'] = 'Vérification de la clé en cours...';
$labels['authentication_succeeded'] = 'Authentication réussie';
$labels['authentication_failed'] = 'Echec d\'authentication';
$labels['authentication_cancelled'] = 'Authentication annulée';
$labels['test_key'] = 'Tester la clé';
$labels['key_checked'] = 'Clé vérifiée avec succès';

// Messages used for the different portions of the plugin
$messages = array();
$messages['successfully_saved'] = 'Paramétrages sauvgardés avec succès.';
