<?php
// Labels used for different portions of the plugin
$labels = array();
$labels['activate'] = 'Activeren';
$labels['twofactor_webauthn'] = '2-staps authenticatie';
$labels['config'] = '2-staps authenticatie';
$labels['two_step_verification_form'] = '2-staps authenticatieformulier';
$labels['add_new'] = 'Beveiligingssleutel toevoegen';
$labels['no_keys_yet'] = 'Nog geen sleutels geregistreerd';
$labels['registered_keys'] = 'Geregistreerde sleutels';
$labels['loading'] = 'Laden...';
$labels['confirm_delete_key'] = 'Sleutel verwijderen met ID';
$labels['key_registered'] = 'Sleutel succesvol geregistreerd';
$labels['key_deleted'] = 'Sleutel verwijderd';
$labels['please_activate_key'] = 'Activeer uw beveiligingssleutel nu';
$labels['verifying_key'] = 'Uw sleutel wordt geverifieerd...';
$labels['authentication_succeeded'] = 'Authenticatie geslaagd';
$labels['authentication_failed'] = 'Authenticatie mislukt';
$labels['authentication_cancelled'] = 'Authenticatie geannuleerd';
$labels['test_key'] = 'Sleutel testen';
$labels['key_checked'] = 'Sleutel gecontroleerd met ID';
$labels['request_key_name'] = 'Voer een naam in voor deze sleutel';
$labels['edit_key_name'] = 'Voer een nieuwe naam in voor sleutel';
$labels['key_renamed'] = 'Sleutel hernoemd naar';
$labels['lock_config'] = 'Configuratie vergrendelen';
$labels['lock_hint'] = 'Vereist authenticatie om 2FA te wijzigen of uit te schakelen.';

// Messages used for the different portions of the plugin
$messages = array();
$messages['successfully_saved'] = 'Instellingen zijn succesvol opgeslagen.';
$messages['error_locked'] = 'Actie mislukt: configuratie is vergrendeld.';
