<?php
// Labels used for different portions of the plugin
$labels = array();
$labels['activate'] = 'Включить';
$labels['twofactor_webauthn'] = 'Двухэтапная аутентификация';
$labels['config'] = 'Двухэтапная аутентификация';
$labels['two_step_verification_form'] = 'Форма двухэтапной аутентификации';
$labels['add_new'] = 'Добавить ключ безопасности';
$labels['no_keys_yet'] = 'Пока что нет зарегестрированных ключей безопасности';
$labels['registered_keys'] = 'Зарегистрированные ключи безопасности';
$labels['loading'] = 'Загрузка...';
$labels['confirm_delete_key'] = 'Уверены, что хотите удалить ключ с ID';
$labels['key_registered'] = 'Ключ успешно зарегистрирован';
$labels['key_deleted'] = 'Ключ удален';
$labels['please_activate_key'] = 'Пожалуйста, активируйте свой ключ сейчас';
$labels['verifying_key'] = 'Проверка ключа...';
$labels['authentication_succeeded'] = 'Аутентификация успешна!';
$labels['authentication_failed'] = 'Аутентификация не удалась';
$labels['authentication_cancelled'] = 'Аутентификация отменена';
$labels['test_key'] = 'Тест ключа';
$labels['key_checked'] = 'Проверка ключа успешна!';

// Messages used for the different portions of the plugin
$messages = array();
$messages['successfully_saved'] = 'Настройки успешно сохранены.';
