<?php
// Labels used for different portions of the plugin
$labels = array();
$labels['activate'] = 'Verwenden';
$labels['twofactor_webauthn'] = 'Zwei-Faktor Authentifizierung';
$labels['config'] = '2-Faktor Authentifizierung';
$labels['two_step_verification_form'] = 'Zwei-Faktor Authentifizierungs Formular';
$labels['add_new'] = 'Sicherheitsschlüssel hinzufügen';
$labels['no_keys_yet'] = 'Bisher keine Schlüssel registriert';
$labels['registered_keys'] = 'Registrierte Sicherheitsschlüssel';
$labels['loading'] = 'Wird geladen...';
$labels['confirm_delete_key'] = 'Soll dieser Schlüssel wirklich gelöscht werden? ID';
$labels['key_registered'] = 'Schlüssel wurde erfolgreich registriert';
$labels['key_deleted'] = 'Schlüssel gelöscht';
$labels['please_activate_key'] = 'Bitte aktiviere jetzt den Sicherheitsschlüssel';
$labels['verifying_key'] = 'Schlüssel wird verifiziert...';
$labels['authentication_succeeded'] = 'Authentifizierung erfolgreich';
$labels['authentication_failed'] = 'Authentifizierung fehlgeschlagen';
$labels['authentication_cancelled'] = 'Authentifizierung abgebrochen';
$labels['test_key'] = 'Schlüssel testen';
$labels['key_checked'] = 'Schlüssel erfolgreich getestet';

// Messages used for the different portions of the plugin
$messages = array();
$messages['successfully_saved'] = 'Einstellungen erfolgreich gespeichert.';
