<?php
// Labels used for different portions of the plugin
$labels = array();
$labels['activate'] = 'Activate';
$labels['twofactor_webauthn'] = 'Two-factor authentication';
$labels['config'] = 'Two-factor authentication';
$labels['two_step_verification_form'] = 'Two-factor authentication form';
$labels['add_new'] = 'Add security key';
$labels['no_keys_yet'] = 'No keys registered yet';
$labels['registered_keys'] = 'Registered security keys';
$labels['loading'] = 'Loading...';
$labels['confirm_delete_key'] = 'Really delete key with ID';
$labels['key_registered'] = 'Key successfully registered';
$labels['key_deleted'] = 'Key deleted';
$labels['please_activate_key'] = 'Please activate your security key now';
$labels['verifying_key'] = 'Verifying key...';
$labels['authentication_succeeded'] = 'Authentication succeeded';
$labels['authentication_failed'] = 'Authentication failed';
$labels['authentication_cancelled'] = 'Authentication cancelled';
$labels['test_key'] = 'Test key';
$labels['key_checked'] = 'Successfully checked key';
$labels['request_key_name'] = 'Please enter a name for this key';
$labels['edit_key_name'] = 'Please enter a new name for key';
$labels['key_renamed'] = 'Key renamed to';
$labels['lock_config'] = 'Lock configuration';
$labels['lock_hint'] = 'Require authentication to change or disable 2FA.';

// Messages used for the different portions of the plugin
$messages = array();
$messages['successfully_saved'] = 'Settings saved successfully.';
$messages['error_locked'] = 'Action failed: configuration is locked.';
