<?php
// Labels used for different portions of the plugin
$labels = array();
$labels['activate'] = 'Активирай';
$labels['twofactor_webauthn'] = 'Двуфакторна идентификация';
$labels['config'] = 'Двуфакторна идентификация';
$labels['two_step_verification_form'] = 'Форма за двуфакторна идентификация';
$labels['add_new'] = 'Добави ключ за безопасност';
$labels['no_keys_yet'] = 'Засега нямате регестрирани ключове за безопасност';
$labels['registered_keys'] = 'Регистрирани ключове за безопасност';
$labels['loading'] = 'Зареждане...';
$labels['confirm_delete_key'] = 'Сигурни ли сте, че искате да изтриете ключ с ID';
$labels['key_registered'] = 'Ключът е успешно регистриран';
$labels['key_deleted'] = 'Ключът е изтрит';
$labels['please_activate_key'] = 'Моля, активирайте своя ключ сега';
$labels['verifying_key'] = 'Проверка на ключа...';
$labels['authentication_succeeded'] = 'Идентификацията е успешна!';
$labels['authentication_failed'] = 'Идентификацията е неуспешна';
$labels['authentication_cancelled'] = 'Идентификацията е отменена';
$labels['test_key'] = 'Тест на ключ';
$labels['key_checked'] = 'Проверката на ключа е успешна!';

// Messages used for the different portions of the plugin
$messages = array();
$messages['successfully_saved'] = 'Настройките са запазени успешно.';
