#twofactor_webauthn_keylist SPAN { font-size: 16px; cursor: pointer; margin-left: 5px; }
#twofactor_webauthn_keylist SPAN.rename { color: orange; }
#twofactor_webauthn_keylist SPAN.delete { color: red; }
